import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate


def show_backend(backend):
    x = PrettyTable(["Title", "ID", "Provider", "State"])
    x.add_row([backend.title, backend.id, backend.provider, backend.info['state']])
    print x
    print

    print "Machines:"
    x = PrettyTable(["Name", "ID", "State", "Public Ips"])
    machines = backend.machines()

    for machine in machines:
        try:
            public_ips = machine.info['public_ips']
            ips = " -- ".join(public_ips)
        except:
            ips = ""
        x.add_row([machine.name, machine.id, machine.info['state'], ips])

    print x


def list_backends(client, pretty):
    backends = client.backends()
    if not client.backends():
        print "No backends found"
        sys.exit(0)

    if pretty:
        x = PrettyTable(["Name", "ID", "Provider", "State"])
        for backend in backends:
            x.add_row([backend.title, backend.id, backend.provider, backend.info['state']])

        print x
    else:
        for backend in backends:
            # print backend.title, backend.id, backend.provider, backend.info['state']
            print "%-40s %-40s %-30s %-20s" % (backend.title, backend.id, backend.provider, backend.info['state'])


def choose_backend(client, args):
    backend_id = args.id
    backend_name = args.name
    if backend_id:
        backends = client.backends(id=backend_id) or None
        backend = backends[0] if backends else None
    elif backend_name:
        backends = client.backends(name=backend_name)
        backend = backends[0] if backends else None
    else:
        backends = client.backends(search=args.backend)
        backend = backends[0] if backends else None

    return backend


def return_backend(client, args):
    backend_id = args.backend_id
    backend_name = args.backend_name
    if backend_id:
        backends = client.backends(id=backend_id) or None
        backend = backends[0] if backends else None
    elif backend_name:
        backends = client.backends(name=backend_name)
        backend = backends[0] if backends else None
    else:
        backends = client.backends(search=args.backend)
        backend = backends[0] if backends else None

    return backend


def add_backend(client, args):
    provider = args.provider
    if "ec2" in provider:
        add_ec2_backend(client, args)
    elif "rackspace" in provider:
        add_rackspace_backend(client, args)
    elif "nepho" in provider:
        add_nepho_backend(client, args)
    elif "digi" in provider:
        add_digital_backend(client,args)
    elif "linode" in provider:
        add_linode_backend(client, args)
    elif "openstack" in provider:
        add_openstack_backend(client, args)
    elif "softlayer" in provider:
        add_softlayer_backend(client, args)
    elif "hp" in provider:
        add_hp_backend(client, args)
    elif "docker" in provider:
        add_docker_backend(client, args)
    elif "azure" in provider:
        add_azure_backend(client, args)
    elif "bare" in provider:
        add_bare_metal_backend(client, args)
    elif "gce" in provider:
        add_gce_backend(client, args)
    elif provider == "vcloud":
        add_vcloud_backend(client, args)
    elif provider == "indonesian_vcloud":
        add_indonesian_backend(client, args)
    elif provider == "libvirt":
        add_libvirt_backend(client, args)


def add_gce_backend(client, args):
    title = args.name
    provider = args.provider

    email = args.gce_email
    project_id = args.gce_project_id
    private_key_path = args.gce_private_key

    if private_key_path:
        with open(private_key_path, 'r') as f:
            private_key = f.read()
    else:
        private_key = ""

    client.add_backend(title=title, provider=provider, email=email, project_id=project_id,
                       private_key=private_key)


def add_libvirt_backend(client, args):
    title = args.name
    provider = args.provider

    machine_hostname = args.libvirt_hostname
    machine_user = args.libvirt_user
    machine_key = args.libvirt_key

    client.add_backend(title=title, provider=provider, machine_hostname=machine_hostname, machine_user=machine_user,
                       machine_key=machine_key)


def add_vcloud_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.vcloud_username
    password = args.vcloud_password
    organization = args.vcloud_organization
    host = args.vcloud_host

    client.add_backend(title=title, provider=provider, username=username, password=password,
                       organization=organization, host=host)


def add_indonesian_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.indonesian_username
    password = args.indonesian_password
    organization = args.indonesian_organization

    client.add_backend(title=title, provider=provider, username=username, password=password,
                       organization=organization)


def add_docker_backend(client, args):
    title = args.name
    provider = args.provider

    docker_host = args.docker_host
    docker_port = args.docker_port
    docker_auth_user = args.docker_auth_user
    docker_auth_password = args.docker_auth_password
    if args.docker_key_file:
        with open(args.docker_key_file, 'r') as f:
            key_file = f.read()
    else:
        key_file = ""
    if args.docker_cert_file:
        with open(args.docker_cert_file, 'r') as f:
            cert_file = f.read()
    else:
        cert_file = ""

    client.add_backend(title=title, provider=provider, docker_host=docker_host, docker_port=docker_port,
                       auth_user=docker_auth_user, auth_password=docker_auth_password, key_file=key_file,
                       cert_file=cert_file)


def add_azure_backend(client, args):
    title = args.name
    provider = args.provider

    subscription_id = args.azure_sub_id
    with open(args.azure_cert_path) as f:
        certificate = f.read()

    client.add_backend(title=title, provider=provider, subscription_id=subscription_id, certificate=certificate)


def add_ec2_backend(client, args):
    title = args.name
    provider = args.provider

    api_key = args.ec2_api_key
    api_secret = args.ec2_api_secret
    region = args.ec2_region

    client.add_backend(title=title, provider=provider, api_key=api_key, api_secret=api_secret, region=region)


def add_rackspace_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.rackspace_username
    api_key = args.rackspace_api_key
    region = args.rackspace_region

    client.add_backend(title=title, provider=provider, username=username, api_key=api_key, region=region)


def add_nepho_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.nepho_username
    password = args.nepho_password

    client.add_backend(title=title, provider=provider, username=username, password=password)


def add_digital_backend(client, args):
    title = args.name
    provider = args.provider

    token = args.digi_token

    client.add_backend(title=title, provider=provider, token=token)


def add_linode_backend(client, args):
    title = args.name
    provider = args.provider

    api_key = args.linode_api_key

    client.add_backend(title=title, provider=provider, api_key=api_key)


def add_openstack_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.openstack_username
    password = args.openstack_password
    auth_url = args.openstack_auth_url
    tenant_name = args.openstack_tenant
    region = args.openstack_region

    client.add_backend(title=title, provider=provider, username=username, password=password,
                       auth_url=auth_url, tenant_name=tenant_name, region=region)


def add_softlayer_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.softlayer_username
    api_key = args.softlayer_api_key

    client.add_backend(title=title, provider=provider, username=username, api_key=api_key)


def add_hp_backend(client, args):
    title = args.name
    provider = args.provider

    username = args.hp_username
    password = args.hp_password
    tenant_name = args.hp_tenant
    region = args.hp_region

    client.add_backend(title=title, provider=provider, username=username, password=password, tenant_name=tenant_name,
                       region=region)


def add_bare_metal_backend(client, args):
    title = args.name
    provider = args.provider

    machine_ip = args.bare_hostname
    machine_user = args.bare_user

    machine_port = args.bare_port
    machine_key = args.bare_ssh_key_id

    client.add_backend(title=title, provider=provider, machine_ip=machine_ip, machine_key=machine_key,
                       machine_port=machine_port, machine_user=machine_user)


def backend_action(args):

    client = authenticate()

    if args.action == 'list-backends':
        pretty = args.pretty
        list_backends(client, pretty)
    elif args.action == 'rename-backend':
        backend = choose_backend(client, args)
        backend.rename(args.new_name)
        print "Renamed backend to %s" % args.new_name
    elif args.action == 'delete-backend':
        backend = choose_backend(client, args)
        if backend:
            backend.delete()
            print "Deleted backend %s" % backend.title
        else:
            print "Could not find backend"
    elif args.action == 'describe-backend':
        backend = choose_backend(client, args)
        if backend:
            show_backend(backend)
        else:
            print "Could not find backend"
    elif args.action == 'add-backend':
        add_backend(client, args)
        print "New backend added"



import os
import ConfigParser
from mistclient import MistClient


def authenticate(module):
    home_path = os.getenv("HOME")
    config_path = os.path.join(home_path, '.mist')
    config = ConfigParser.ConfigParser()

    mist_uri = module.params.get('mist_uri')
    mist_email = module.params.get('mist_email')
    mist_password = module.params.get('mist_password')

    # Set mist uri
    config.add_section("mist.io")
    if not mist_uri:
        config.set("mist.io", "mist_uri", "https://mist.io")
    else:
        config.set("mist.io", "mist_uri", mist_uri)

    # Set default credentials
    config.add_section("mist.credentials")
    config.set("mist.credentials", "email", None)
    config.set("mist.credentials", "password", None)

    # Read configuration file
    if os.path.isfile(config_path):
            config.readfp(open(config_path))

    mist_uri = config.get("mist.io", "mist_uri")
    if not mist_email:
        mist_email = config.get("mist.credentials", "email") or ""

    if not mist_password:
        mist_password = config.get("mist.credentials", "password") or ""

    return init_client(mist_uri, mist_email, mist_password)


def init_client(mist_uri="https://mist.io", email=None, password=None):
    client = MistClient(mist_uri, email, password)
    return client


import sys

from prettytable import PrettyTable
from mistcommand.helpers.backends import return_backend
from mistcommand.helpers.login import authenticate


def list_locations(backend, pretty):
    locations = backend.locations

    if pretty:
        x = PrettyTable(["Name", "ID"])
        for location in locations:
            x.add_row([location['name'], location['id']])

        print x
    else:
        for location in locations:
            print "%-20s %-20s" % (location['name'], location['id'])


def location_action(args):

    if args.action == 'list-locations':
        client = authenticate()
        backend = return_backend(client, args)
        pretty = args.pretty

        list_locations(backend, pretty)


import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate
from mistcommand.helpers.backends import return_backend


def list_images(backend, search_term, pretty):
    if not search_term:
        images = []
        for image in backend.images:
            if image['star']:
                images.append(image)
    elif search_term == "all":
        images = backend.images
    else:
        images = backend.search_image(search_term)

    if pretty:
        x = PrettyTable(["Name", "ID"])
        for image in images:
            x.add_row([image['name'], image['id']])

        print x
    else:
        for image in images:
            print "%-40s %-40s" % (image['name'], image['id'])


def image_action(args):

    if args.action == 'list-images':
        client = authenticate()
        backend = return_backend(client, args)
        search_term = args.search
        pretty = args.pretty

        list_images(backend, search_term, pretty)


import json
import os
import re

from time import time

from mistclient.helpers import RequestsHandler


class Backend(object):
    """
    A backend instance.
    """
    def __init__(self, backend, mist_client):
        """

        :param backend: Dict that is returned from the API, with all the available info.
        :param mist_client: The MistClient instance that initiated the creation of this Backend instance.
        :returns: A Backend object.
        """
        self.mist_client = mist_client
        self.info = backend
        self.title = backend['title']
        self.id = backend['id']
        self.enabled = backend['enabled']
        self.provider = backend['provider']
        self.api_token = self.mist_client.api_token

        self._machines = None

    def __str__(self):
        return "%s => %s, %s, %s" % (self.__class__.__name__, self.title, self.provider, self.id)

    def __repr__(self):
        return "%s => %s, %s, %s" % (self.__class__.__name__, self.title, self.provider, self.id)

    def request(self, *args, **kwargs):
        """
        The main purpose of this is to be a wrapper-like function to pass the api_token and all the other params to the
        requests that are being made

        :returns: An instance of RequestsHandler
        """
        return RequestsHandler(*args, api_token=self.api_token, **kwargs)

    def delete(self):
        """
        Delete the backend from the list of added backends in mist.io service.

        :returns: A list of mist.clients' updated backends.
        """
        req = self.request(self.mist_client.uri + '/backends/' + self.id)
        req.delete()
        self.mist_client.update_backends()

    def rename(self, new_name):
        """
        Rename the backend.

        :param new_name: New name for the backend to be renamed to.
        :returns: A list of mist.clients' updated backends.
        """
        payload = {
            'new_name': new_name
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri + '/backends/' + self.id, data=data)
        req.put()
        self.title = new_name
        self.mist_client.update_backends()

    def enable(self):
        """
        Enable the Backend.

        :returns:  A list of mist.clients' updated backends.
        """
        payload = {
            "new_state": "1"
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+'/backends/'+self.id, data=data)
        req.post()
        self.enabled = True
        self.mist_client.update_backends()

    def disable(self):
        """
        Disable the Backend.

        :returns:  A list of mist.clients' updated backends.
        """
        payload = {
            "new_state": "0"
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+'/backends/'+self.id, data=data)
        req.post()
        self.enabled = False
        self.mist_client.update_backends()

    @property
    def sizes(self):
        """
        Available machine sizes to be used when creating a new machine.

        :returns: A list of available machine sizes.
        """
        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/sizes')
        sizes = req.get().json()
        return sizes

    @property
    def locations(self):
        """
        Available locations to be used when creating a new machine.

        :returns: A list of available locations.
        """
        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/locations')
        locations = req.get().json()
        return locations

    @property
    def networks(self):
        """
        Available networks.

        :returns: A list of available networks associated to a provider.
        """
        if self.provider in ['openstack', 'nephoscale']:
            req = self.request(self.mist_client.uri+'/backends/'+self.id+'/networks')
            networks = req.get().json()
            return networks
        else:
            print "Network actions not supported yet for %s provider" % self.provider

    @property
    def images(self):
        """
        Available images to be used when creating a new machine.

        :returns: A list of all available images.
        """
        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/images')
        images = req.get().json()
        return images

    def search_image(self, search_term):
        """
        Search for a specific image by providing a search term (mainly used with ec2's community and public images)

        :param search_term: Search term to be used when searching for images's names containing this term.
        :returns: A list of all images, whose names contain the given search_term.
        """
        payload = {
            'search_term': search_term
        }
        data = json.dumps(payload)

        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/images', data=data)
        images = req.get().json()
        return images

    def _list_machines(self):
        """
        Request a list of all added machines.

        Populates self._machines dict with mist.client.model.Machine instances
        """
        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/machines')
        machines = req.get().json()

        if machines:
            for machine in machines:
                self._machines[machine['id']] = Machine(machine, self)
        else:
            self._machines = {}

    def machines(self, id=None, name=None, search=None):
        """
        Property-like function to call the _list_machines function in order to populate self._machines dict

        :returns: A list of Machine instances.
        """
        if self._machines is None:
            self._machines = {}
            self._list_machines()

        if id:
            return [self._machines[machine_id] for machine_id in self._machines.keys()
                    if id == self._machines[machine_id].id]
        elif name:
            return [self._machines[machine_id] for machine_id in self._machines.keys()
                    if name == self._machines[machine_id].name]
        elif search:
            return [self._machines[machine_id] for machine_id in self._machines.keys()
                    if search in self._machines[machine_id].name
                    or search in self._machines[machine_id].id]
        else:
            return [self._machines[machine_id] for machine_id in self._machines.keys()]

    def update_machines(self):
        """
        Update added machines' info and re-populate the self._machines dict.

        This one is used whenever a new machine is created, rebooted etc etc or whenever you want to update the list
        of added machines.

        :returns: A list of Machine instances.
        """
        self._machines = {}
        self._list_machines()
        return self._machines

    def create_machine(self, name, key, image_id, location_id, size_id, 
                       image_extra="", disk="", script="", monitoring=False, 
                       ips=[], networks=[], location_name=""):
        """
        Create a new machine on the given backend

        :param name: Name of the machine
        :param key: Key Object to associate with the machine
        :param image_id: Id of image to be used with the creation
        :param location_id: Id of the backend's location to create the machine
        :param size_id: If of the size of the machine
        :param image_extra: Needed only by Linode backend
        :param disk: Needed only by Linode backend
        :returns: An update list of added machines
        """
        payload = {
            'name': name,
            'key': key.id,
            'image': image_id,
            'location': location_id,
            'size': size_id,
            'image_extra': image_extra,
            'disk': disk,
            'script': script,
            'monitoring': monitoring,
            'ips': ips,
            'networks': networks,
            'location_name': location_name
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+'/backends/'+self.id+'/machines', data=data)
        req.post()
        self.update_machines()


class Machine(object):
    """
    A machine instance
    """
    def __init__(self, machine, backend):
        """

        :param machine: Dict with all available info for the machine
        :param backend: The Backend instance which initiates the new machine
        :returns: A machine instance
        """
        self.backend = backend
        self.mist_client = backend.mist_client
        self.info = machine
        self.api_token = self.mist_client.api_token
        self.name = machine['name']
        self.id = machine['id']
        self.probed = None

    def __str__(self):
        return "%s => %s, %s" % (self.__class__.__name__, self.name, self.id)

    def __repr__(self):
        return "%s => %s, %s" % (self.__class__.__name__, self.name, self.id)

    def request(self, *args, **kwargs):
        """
        The main purpose of this is to be a wrapper-like function to pass the api_token and all the other params to the
        requests that are being made

        :returns: An instance of RequestsHandler
        """
        return RequestsHandler(*args, api_token=self.api_token, **kwargs)

    def _machine_actions(self, action):
        """
        Actions for the machine (e.g. stop, start etc)

        :param action: Can be "reboot", "start", "stop", "destroy"
        :returns: An updated list of the added machines
        """
        payload = {
            'action': action
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+'/backends/'+self.backend.id+'/machines/'+self.id, data=data)
        req.post()
        self.backend.update_machines()

    def reboot(self):
        """
        Reboot machine
        """
        self._machine_actions("reboot")

    def start(self):
        """
        Start a stopped machine
        """
        self._machine_actions("start")

    def stop(self):
        """
        Stop a running machine
        """
        self._machine_actions("stop")

    def destroy(self):
        """
        Destroy machine
        """
        self._machine_actions("destroy")

    def probe(self, key_id=None, ssh_user=None):
        """
        If no parameter is provided, mist.io will try to probe the machine with the default
        :param key_id: Optional. Give if you explicitly want to probe with this key_id
        :param ssh_user: Optional. Give if you explicitly want a specific user
        :returns: A list of data received by the probing (e.g. uptime etc)
        """
        ips = [ip for ip in self.info['public_ips'] if ':' not in ip]

        if not ips:
            raise Exception("No public IPv4 address available to connect to")
        payload = {
            'host': ips[0],
            'key': key_id,
            'ssh_user': ssh_user
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/probe", data=data)
        probe_info = req.post().json()
        self.probed = True
        return probe_info

    def associate_key(self, key_id, host=None, ssh_user=None, ssh_port=22):
        payload = {
            'ssh_user': ssh_user,
            'host': host,
            'ssh_port': ssh_port
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/keys/"+key_id,
                           data=data)
        req.put()
        self.mist_client.update_keys()

    def _toggle_monitoring(self, action, no_ssh=False):
        """
        Enable or disable monitoring on a machine

        :param action: Can be either "enable" or "disable"
        """
        payload = {
            'action': action,
            'name': self.name,
            'no_ssh': no_ssh,
            'public_ips': self.info['public_ips'],
            'dns_name': self.info['extra'].get('dns_name', 'n/a')
        }

        data = json.dumps(payload)

        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/monitoring",
                           data=data)
        req.post()

    def enable_monitoring(self, no_ssh=False):
        """
        Enable monitoring
        """
        return self._toggle_monitoring(action="enable", no_ssh=no_ssh)

    def disable_monitoring(self, no_ssh=False):
        """
        Disable monitoring
        """
        return self._toggle_monitoring(action="disable", no_ssh=no_ssh)

    def get_stats(self, start=int(time()), stop=int(time())+10, step=10):
        """
        Get stats of a monitored machine

        :param start: Time formatted as integer, from when to fetch stats (default now)
        :param stop: Time formatted as integer, until when to fetch stats (default +10 seconds)
        :param step: Step to fetch stats (default 10 seconds)
        :returns: A dict of stats
        """
        payload = {
            'v': 2,
            'start': start,
            'stop': stop,
            'step': step
        }

        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/stats", data=data)
        stats = req.get().json()
        return stats

    @property
    def available_metrics(self):
        """
        List all available metrics that you can add to this machine

        :returns: A list of dicts, each of which is a metric that you can add to a monitored machine
        """
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/metrics")
        metrics = req.get().json()
        return metrics

    def add_metric(self, metric_id):
        """
        Add a metric to a monitored machine

        :param metric_id: Metric_id (provided by self.available_metrics)
        """
        payload = {
            'metric_id': metric_id
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/metrics", data=data)
        req.put()

    def remove_metric(self, metric_id):
        """
        Remove a metric from a monitored machine

        :param metric_id: Metric_id (provided by self.get_stats() )
        """
        payload = {
            'metric_id': metric_id
        }

        data = json.dumps(payload)

        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/metrics", data=data)
        req.delete()

    def add_python_plugin(self, name, python_file, value_type="gauge", unit=None):
        """
        Add a custom python plugin to the collectd instance of a monitored plugin

        :param python_file: Path of the python file to be added as custom python plugin
        :param name: Name of the plugin
        :param value_type: Optional. Can be either "gauge" or "derive"
        :param unit: Optional. If given the new plugin will be measured according to this unit
        """

        if not os.path.isfile(python_file):
            raise Exception(python_file, "is not a file or could not be found in tho given path")

        with open(python_file) as f:
            script = f.read()

        payload = {
            'plugin_type': 'python',
            'name': name,
            'unit': unit,
            'value_type': value_type,
            'read_function': script,
            'host': self.info['public_ips'][0]
        }

        data = json.dumps(payload)

        #PLugin id must be in lowercase
        plugin_id = name.lower()

        #PLugin id must contain only alphanumeric chars
        pattern = re.compile('\W')
        plugin_id = re.sub(pattern, "_", plugin_id)

        #Plugin id should not have double underscores
        while "__" in plugin_id:
            pattern = "\r?__"
            plugin_id = re.sub(pattern, "_", plugin_id)

        #Plugin id should not have underscore as first or last char
        if plugin_id[-1] == "_":
            plugin_id = plugin_id[:-2]

        if plugin_id[0] == "_":
            plugin_id = plugin_id[1:]

        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/plugins/"+plugin_id,
                           data=data)
        req.post()

    def tag(self, tag):
        payload = {
            'tag': tag
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+self.backend.id+"/machines/"+self.id+"/metadata",
                           data=data)
        req.post()
        self.backend.update_machines()


class Key(object):
    """
    A Key instance
    """
    def __init__(self, key, mist_client):
        """

        :param key: A dict with all the available info for this key
        :param mist_client: The MistClient instance that initiated the creation of this Key instance.
        :returns: A Key instance
        """
        self.mist_client = mist_client
        self.api_token = self.mist_client.api_token
        self.id = key['id']
        self.is_default = key['isDefault']
        self.info = key

    def __str__(self):
        return "%s => %s" % (self.__class__.__name__, self.id)

    def __repr__(self):
        return "%s => %s" % (self.__class__.__name__, self.id)

    def request(self, *args, **kwargs):
        """
        The main purpose of this is to be a wrapper-like function to pass the api_token and all the other params to the
        requests that are being made

        :returns: An instance of RequestsHandler
        """
        return RequestsHandler(*args, api_token=self.api_token, **kwargs)

    @property
    def private(self):
        """
        Return the private ssh-key

        :returns: The private ssh-key as string
        """
        req = self.request(self.mist_client.uri+'/keys/'+self.id+"/private")
        private = req.get().json()
        return private

    @property
    def public(self):
        """
        Return the public ssh-key

        :returns: The public ssh-key as string
        """
        req = self.request(self.mist_client.uri+'/keys/'+self.id+"/public")
        public = req.get().json()
        return public

    def rename(self, new_name):
        """
        Rename a key

        :param new_name: New name for the key (will also serve as the key's id)
        :returns: An updated list of added keys
        """
        payload = {
            'new_id': new_name
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+'/keys/'+self.id, data=data)
        req.put()
        self.id = new_name
        self.mist_client.update_keys()

    def set_default(self):
        """
        Set this key as the default key

        :returns: An updated list of added keys
        """
        req = self.request(self.mist_client.uri+'/keys/'+self.id)
        req.post()
        self.is_default = True
        self.mist_client.update_keys()

    def delete(self):
        """
        Delete this key from mist.io

        :returns: An updated list of added keys
        """
        req = self.request(self.mist_client.uri+'/keys/'+self.id)
        req.delete()
        self.mist_client.update_keys()

    def associate_to_machine(self, backend_id, machine_id, host=None, ssh_user=None, ssh_port=22):
        payload = {
            'ssh_user': ssh_user,
            'host': host,
            'ssh_port': ssh_port
        }
        data = json.dumps(payload)
        req = self.request(self.mist_client.uri+"/backends/"+backend_id+"/machines/"+machine_id+"/keys/"+self.id,
                           data=data)
        req.put()
        self.mist_client.update_keys()



import os
import sys
import tempfile
import shutil

from mistcommand.helpers.login import authenticate
from mistcommand.helpers.machines import choose_machines_by_tag


def update_key_association(client, machines):
    keys = client.keys()
    machine_ids = [machine.id for machine in machines]
    assoc_keys = []

    for key in keys:
        assoc_machines = key.info['machines']
        for assoc_machine in assoc_machines:
            id = assoc_machine[1]
            if id in machine_ids:
                for machine in machines:
                    if machine.id == id:
                        print "Found key association for machine: %s" % machine.name
                        machine.info['assoc'] = {
                            'key_id': key.id,
                            'user': assoc_machine[3],
                            'port': assoc_machine[-1]
                        }
                        assoc_keys.append(key)

    return machines, assoc_keys


def init_ansible_tmp_dir(assoc_machines, assoc_keys, tag):
    tmp_dir = tempfile.mkdtemp()
    tmp_ansible_host_file = os.path.join(tmp_dir, 'mist_ansible_hosts')

    # Save the ssh private keys
    for key in assoc_keys:
        key_file = os.path.join(tmp_dir, key.id)
        with open(key_file, 'w') as f:
            os.chmod(key_file, 0600)
            f.write(key.private)
            f.write("\nPAPARIA")

    with open(tmp_ansible_host_file, 'w') as f:
        f.write("[%s]" % tag)

    for machine in assoc_machines:
        ips = [ip for ip in machine.info['public_ips'] if ':' not in ip]
        if not ips:
            break
        ip = ips[0]
        user = machine.info['assoc']['user']
        port = machine.info['assoc']['port']
        key_file = os.path.join(tmp_dir, machine.info['assoc']['key_id'])

        with open(tmp_ansible_host_file, 'a') as f:
            f.write(
                "\n%s\tansible_ssh_host=%s\tansible_ssh_user=%s\tansible_ssh_private_key_file=%s\tansible_ssh_port=%d\n" %
                (machine.name, ip, user, key_file, port)

            )

    return tmp_dir, tmp_ansible_host_file


def parse_output(result, assoc_machines):
    for machine in assoc_machines:
        print "\nFinished in machine: %s" % machine.name
        machine_output = result['contacted'][machine.name]
        failed = machine_output.get("failed", False)

        if failed:
            print "FAILED"
            msg = machine_output.get('msg', None)
            if msg:
                print "Error output: \n%s" % msg
        else:
            stdout = machine_output.get('stdout', None)
            if stdout:
                print "Output: \n%s" % stdout


def ansible_action(args, Runner, Inventory):
    client = authenticate()
    command = args.command
    tag = args.tag

    machines = choose_machines_by_tag(client, tag)
    if not machines:
        print "No machines found with tag: %s" % tag
        sys.exit(0)
    else:
        print "Found tagged machines"

    machines, assoc_keys = update_key_association(client, machines)
    assoc_machines = [machine for machine in machines if machine.info.get('assoc', None)]

    if not assoc_machines:
        print "Could not find machines with associated keys"
        sys.exit(0)

    tmp_dir, tmp_ansible_host_file = init_ansible_tmp_dir(assoc_machines, assoc_keys, tag)

    inventory = Inventory(host_list=tmp_ansible_host_file)

    runner = Runner(module_name='command', module_args=command,  inventory=inventory, host_list=[tag])
    result = runner.run()

    parse_output(result, assoc_machines)
    shutil.rmtree(tmp_dir)



import sys

try:
    import requests
except ImportError:
    print "requests required for this module'"
    sys.exit(1)


class RequestsHandler(object):
    """
    A wrapper-like class to be used with all the requests being made to mist.io service.
    """

    def __init__(self, mist_uri, data=None, api_token=None, timeout=None, api_version=None):
        """

        :param mist_uri: The uri to make the requests to.
        :param data: Json object with all the params needed by some requests.
        :param api_token: If api_token is used, then you do not have to provide username and password.
        :param timeout: Optional. If given the request will fail if it lasts longer than the timeout.

        :returns: A RequestsHandler instance.
        """
        self.headers = {'Authorization': api_token}
        if api_version:
            self.headers['Api-Version'] = api_version
        self.uri = mist_uri
        self.data = data
        self.timeout = timeout

    def response(self, resp):
        """
        For each respone we check if the response status is ok, otherwise raise an error with the the response's
        content.
        :param resp: The response we got after a request.
        :return: Return the response if response.ok, otherwise raise error with the response.content.
        """
        if resp.ok:
            return resp
        else:
            raise Exception(resp.content)

    def post(self):
        resp = requests.post(self.uri, data=self.data, headers=self.headers, timeout=self.timeout)
        return self.response(resp)

    def get(self):
        resp = requests.get(self.uri, data=self.data, headers=self.headers, timeout=self.timeout)
        return self.response(resp)

    def put(self):
        resp = requests.put(self.uri, data=self.data, headers=self.headers, timeout=self.timeout)
        return self.response(resp)

    def delete(self):
        resp = requests.delete(self.uri, data=self.data, headers=self.headers, timeout=self.timeout)
        return self.response(resp)


def machine_from_id(backend, id):
    machines = backend.machines
    for key in machines.keys():
        machine = machines[key]
        if id == machine.id:
            return machine
    return None


def backend_from_id(client, id):
    backends = client.backends
    for key in backends.keys():
        backend = backends[key]
        if id == backend.id:
            return backend
    return


import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate


def show_key(key):
    print "Name: %s" % key.id
    print

    print "Private key:"
    print key.private
    print

    print "Puclic key:"
    print key.public


def list_keys(client, pretty):
    keys = client.keys()
    if not keys:
        print "No keys found"
        sys.exit(0)

    if pretty:
        x = PrettyTable(["Name", "Is Default"])
        for key in keys:
            x.add_row([key.id, key.is_default])

        print x
    else:
        for key in keys:
            print "%s" % key.id


def key_action(args):

    client = authenticate()

    if args.action == 'add-key':
        name = args.name
        key_path = args.key_path
        auto = args.auto_generate

        if auto:
            private = client.generate_key()
        else:
            with open(key_path, "r") as f:
                private = f.read().strip("\n")

        client.add_key(key_name=name, private=private)
        print "Added key %s" % name
    elif args.action == 'list-keys':
        pretty = args.pretty
        list_keys(client, pretty)
    elif args.action == 'delete-key':
        key_id = args.key_name if args.key_name else args.key_id
        keys = client.keys(id=key_id)
        key = keys[0] if keys else None
        if key:
            key.delete()
            print "Deleted %s" % key.id
        else:
            print "Cound not find key: %s" % args.key
            sys.exit(0)
    elif args.action == 'rename-key':
        key_name = args.key_name if args.key_name else args.key_id
        new_name = args.new_name

        keys = client.keys(id=key_name)
        key = keys[0] if keys else None

        if key:
            key.rename(new_name)
            print "Renamed key to %s" % new_name
        else:
            print "Could not find key: %s" % key_name
    elif args.action == 'describe-key':
        key_name = args.key_name if args.key_name else args.key_id
        keys = client.keys(id=key_name)
        key = keys[0] if keys else None
        if key:
            show_key(key)
        else:
            print "Could not find key: %s" % key_name




import sys

from prettytable import PrettyTable
from mistcommand.helpers.backends import return_backend
from mistcommand.helpers.login import authenticate


def list_networks(backend, pretty):
    if backend.provider not in ['openstack']:
        print "Network action not supported yet for %s provider" % backend.provider
        sys.exit(0)

    networks = backend.networks

    for network in networks:
        print "%-20s %-40s %-15s" % (network['name'], network['id'], network['status'])


def network_action(args):
    if args.action == 'list-networks':
        client = authenticate()
        backend = return_backend(client, args)
        pretty = args.pretty

        list_networks(backend, pretty)


import json

from mistclient.helpers import RequestsHandler
from mistclient.model import Backend, Key


class MistClient(object):
    """
    The base class that initiates a new client that connects with mist.io service.
    """
    def __init__(self, mist_uri="https://mist.io", email=None, password=None):
        """
        Initialize the mist.client. In case email and password are given, it will try to authenticate with mist.io
        and keep the api_token that is returned to be used with the later requests.


        :param mist_uri: By default it is 'https://mist.io'. Can be changed if there's a different installation of mist.io
        :param email: Email to authenticate with mist.io. May be left 'None' if there's a standalone installation of mist.io that does not require authentication.
        :param password: Password to authenticate with mist.io. May be left 'None' if there's a standalone installation of mist.io that does not require authentication.
        """
        self.uri = mist_uri
        self.email = email
        self.password = password
        self.api_token = None
        self.user_details = None

        self._backends = None
        self._machines = None
        self._keys = None

        if self.email and self.password:
            self.__authenticate()

    def __authenticate(self):
        """
        Sends a json payload with the email and password in order to get the authentication api_token to be used with
        the rest of the requests
        """
        payload = {
            'email': self.email,
            'password': self.password
        }

        data = json.dumps(payload)
        req = self.request(self.uri+'/auth', data=data)
        response = req.post().json()
        token = response['mist_api_token']
        self.api_token = "mist_1 %s:%s" % (self.email, token)
        self.user_details = response

    def request(self, *args, **kwargs):
        """
        The main purpose of this is to be a wrapper-like function to pass the api_token and all the other params to the
        requests that are being made

        :returns: An instance of mist.client.helpers.RequestsHandler
        """
        return RequestsHandler(*args, api_token=self.api_token, **kwargs)

    @property
    def supported_providers(self):
        """
        Request a list of all available providers

        :returns: A list of all available providers (e.g. {'provider': 'ec2_ap_northeast', 'title': 'EC2 AP NORTHEAST'})
        """
        req = self.request(self.uri+'/providers', api_version=2)
        providers = req.get().json()
        supported_providers = providers['supported_providers']
        return supported_providers

    def _list_backends(self):
        """
        Request a list of all added backends.

        Populates self._backends dict with mist.client.model.Backend instances
        """
        req = self.request(self.uri+'/backends')
        backends = req.get().json()
        if backends:
            for backend in backends:
                self._backends[backend['id']] = Backend(backend, self)
        else:
            self._backends = {}

    def backends(self, id=None, name=None, provider=None, search=None):
        """
        Property-like function to call the _list_backends function in order to populate self._backends dict

        :returns: A list of Backend instances.
        """
        if self._backends is None:
            self._backends = {}
            self._list_backends()

        if id:
            return [self._backends[backend_id] for backend_id in self._backends.keys()
                    if id == self._backends[backend_id].id]
        elif name:
            return [self._backends[backend_id] for backend_id in self._backends.keys()
                    if name == self._backends[backend_id].title]
        elif provider:
            return [self._backends[backend_id] for backend_id in self._backends.keys()
                    if provider == self._backends[backend_id].provider]
        elif search:
            return [self._backends[backend_id] for backend_id in self._backends.keys()
                    if search in self._backends[backend_id].title
                    or search in self._backends[backend_id].id
                    or search in self._backends[backend_id].provider]
        else:
            return [self._backends[backend_id] for backend_id in self._backends.keys()]

    def update_backends(self):
        """
        Update added backends' info and re-populate the self._backends dict.

        This one is used whenever a new backend is added, renamed etc etc or whenever you want to update the list
        of added backends.

        :returns: A list of Backend instances.
        """
        self._backends = {}
        self._list_backends()
        return self._backends

    def add_backend(self, title, provider, **kwargs):
        payload = {}
        if provider == "ec2":
            payload = self._add_backend_ec2(**kwargs)
        elif provider == "rackspace":
            payload = self._add_backend_rackspace(**kwargs)
        elif provider == "nephoscale":
            payload = self._add_backend_nephoscale(**kwargs)
        elif provider == "softlayer":
            payload = self._add_backend_softlayer(**kwargs)
        elif provider == "digitalocean":
            payload = self._add_backend_digitalocean(**kwargs)
        elif provider == "gce":
            payload = self._add_backend_gce(**kwargs)
        elif provider == "azure":
            payload = self._add_backend_azure(**kwargs)
        elif provider == "linode":
            payload = self._add_backend_linode(**kwargs)
        elif provider == "bare_metal":
            payload = self._add_backend_bare_metal(**kwargs)
        elif provider in ['vcloud', 'indonesian_vcloud']:
            payload = self._add_backend_vcloud(**kwargs)
        elif provider == "docker":
            payload = self._add_backend_docker(**kwargs)
        elif provider == "libvirt":
            payload = self._add_backend_libvirt(**kwargs)
        elif provider == "hpcloud":
            payload = self._add_backend_hp(**kwargs)
        elif provider == "openstack":
            payload = self._add_backend_openstack(**kwargs)

        payload['title'] = title
        payload['provider'] = provider
        req = self.request(self.uri+'/backends', data=json.dumps(payload), api_version=2)
        response = req.post()
        self.update_backends()
        return

    def _add_backend_rackspace(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'api_key': kwargs.get('api_key', ''),
            'region': kwargs.get('region', '')
        }
        return payload

    def _add_backend_ec2(self, **kwargs):
        payload = {
            'api_key': kwargs.get('api_key', ''),
            'api_secret': kwargs.get('api_secret', ''),
            'region': kwargs.get('region', '')
        }
        return payload

    def _add_backend_nephoscale(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'password': kwargs.get('password', '')
        }
        return payload

    def _add_backend_softlayer(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'api_key': kwargs.get('api_key', '')
        }
        return payload

    def _add_backend_digitalocean(self, **kwargs):
        payload = {
            'token': kwargs.get('token', '')
        }
        return payload

    def _add_backend_gce(self, **kwargs):
        payload = {
            'email': kwargs.get('email', ''),
            'private_key': kwargs.get('private_key', ''),
            'project_id': kwargs.get('project_id', '')
        }
        return payload

    def _add_backend_azure(self, **kwargs):
        payload = {
            'subscription_id': kwargs.get('subscription_id', ''),
            'certificate': kwargs.get('certificate', '')
        }
        return payload

    def _add_backend_linode(self, **kwargs):
        payload = {
            'api_key': kwargs.get('api_key', '')
        }
        return payload

    def _add_backend_bare_metal(self, **kwargs):
        payload = {
            'machine_ip': kwargs.get('machine_ip', ''),
            'machine_key': kwargs.get('machine_key', ''),
            'machine_user': kwargs.get('machine_user', 'root'),
            'machine_port': kwargs.get('machine_port', 22)
        }
        return payload

    def _add_backend_vcloud(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'password': kwargs.get('password', ''),
            'organization': kwargs.get('organization', ''),
            'host': kwargs.get('host', '')
        }
        return payload

    def _add_backend_docker(self, **kwargs):
        payload = {
            'docker_port': int(kwargs.get('docker_port', 4243)),
            'docker_host': kwargs.get('docker_host', ''),
            'auth_user': kwargs.get('auth_user', ''),
            'auth_password': kwargs.get('auth_password', ''),
            'key_file': kwargs.get('key_file', ''),
            'cert_file': kwargs.get('cert_file', '')
        }
        return payload

    def _add_backend_libvirt(self, **kwargs):
        payload = {
            'machine_hostname': kwargs.get('machine_hostname', ''),
            'machine_user': kwargs.get('machine_user', 'root'),
            'machine_key': kwargs.get('machine_key', '')
        }
        return payload

    def _add_backend_hp(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'password': kwargs.get('password', ''),
            'tenant_name': kwargs.get('tenant_name', ''),
            'region': kwargs.get('region', '')
        }
        return payload

    def _add_backend_openstack(self, **kwargs):
        payload = {
            'username': kwargs.get('username', ''),
            'password': kwargs.get('password', ''),
            'auth_url': kwargs.get('auth_url', ''),
            'tenant_name': kwargs.get('tenant_name', ''),
            'region': kwargs.get('region', ''),
            'compute_endpoint': kwargs.get('compute_endpoint', '')
        }
        return payload

    def _list_keys(self):
        """
        Retrieves a list of all added Keys and populates the self._keys dict with Key instances

        :returns: A list of Keys instances
        """
        req = self.request(self.uri+'/keys')
        keys = req.get().json()
        if keys:
            self._keys = {}
            for key in keys:
                self._keys[key['id']] = Key(key, self)
        else:
            self._keys = {}

    def keys(self, id=None, search=None):
        """
        Property-like function to call the _list_keys function in order to populate self._keys dict

        :returns: A list of Key instances
        """
        if self._keys is None:
            self._keys = {}
            self._list_keys()

        if id:
            return [self._keys[key_id] for key_id in self._keys.keys()
                    if id == self._keys[key_id].id]
        elif search:
            return [self._keys[key_id] for key_id in self._keys.keys()
                    if search in self._keys[key_id].id]
        else:
            return [self._keys[key_id] for key_id in self._keys.keys()]

    def update_keys(self):
        """
        Update added keys' info and re-populate the self._keys dict.

        This one is used whenever a new key is added, renamed etc etc or whenever you want to update the list
        of added keys.

        :returns: A list of Key instances.
        """
        self._keys = {}
        self._list_keys()
        return self._keys

    def generate_key(self):
        """
        Ask mist.io to randomly generate a private ssh-key to be used with the creation of a new Key

        :returns: A string of a randomly generated ssh private key
        """
        req = self.request(self.uri+"/keys")
        private_key = req.post().json()
        return private_key['priv']

    def add_key(self, key_name, private):
        """
        Add a new key to mist.io

        :param key_name: Name of the new key (it will be used as the key's id as well).
        :param private: Private ssh-key in string format (see also generate_key() ).

        :returns: An updated list of added keys.
        """
        payload = {
            'id': key_name,
            'priv': private
        }

        data = json.dumps(payload)

        req = self.request(self.uri+'/keys', data=data)
        req.put()
        self.update_keys()

    def _list_machines(self):
        self._machines = []
        for backend in self.backends():
            machines = backend.machines()
            for machine in machines:
                self._machines.append(machine)

    def machines(self, id=None, name=None, search=None):
        if self._machines is None:
            self._list_machines()

        if id:
            return [machine for machine in self._machines if machine.id == id]
        elif name:
            return [machine for machine in self._machines if machine.name == name]
        elif search:
            return [machine for machine in self._machines
                    if search in machine.id or search in machine.name]
        else:
            return self._machines


import os
import sys
import getpass

from yaml import load
from yaml.scanner import ScannerError

from mistclient import MistClient
from mistclient.helpers import machine_from_id, backend_from_id


def load_dbyaml(path):
    if not os.path.isfile(path):
        print "You need to provide a valid path for a db.yaml file"
        sys.exit(1)

    with open(path, 'r') as db_yaml:
        try:
            print ">>>Loading %s" %path
            print
            user_dict = load(db_yaml) or {}
            return user_dict
        except ScannerError:
            print "Error parsing %s" % path
            print "Maybe not a yaml file"
            sys.exit(1)


def init_client():
    print ">>>Log in to mist.io"
    email = raw_input("Email: ")
    password = getpass.getpass("Password: ")
    print
    try:
        client = MistClient(email=email, password=password)
        client.backends
        return client
    except Exception as e:
        print e
        sys.exit(1)


def add_backend(client, backend):
    title = backend.get('title')
    provider = backend.get('provider')
    key = backend.get('apikey', "")
    secret = backend.get('apisecret', "")
    tenant_name = backend.get('tenant_name', "")
    region = backend.get('region', "")
    apiurl = backend.get('apiurl', "")
    compute_endpoint = backend.get('compute_endpoint', None)
    machine_ip = backend.get('machine_ip', None)
    machine_key = backend.get('machine_key', None)
    machine_user = backend.get('machine_user', None)
    machine_port = backend.get('machine_port', None)
    client.add_backend(title, provider, key, secret, tenant_name=tenant_name, region=region, apiurl=apiurl,
                       machine_ip=machine_ip, machine_key=machine_key, machine_user=machine_user,
                       compute_endpoint=compute_endpoint, machine_port=machine_port)


def add_bare_metal_backend(client, backend, keys):
    """
    Black magic is happening here. All of this wil change when we sanitize our API, however, this works until then
    """
    title = backend.get('title')
    provider = backend.get('provider')
    key = backend.get('apikey', "")
    secret = backend.get('apisecret', "")
    tenant_name = backend.get('tenant_name', "")
    region = backend.get('region', "")
    apiurl = backend.get('apiurl', "")
    compute_endpoint = backend.get('compute_endpoint', None)
    machine_ip = backend.get('machine_ip', None)
    machine_key = backend.get('machine_key', None)
    machine_user = backend.get('machine_user', None)
    machine_port = backend.get('machine_port', None)

    if provider == "bare_metal":
        machine_ids = backend['machines'].keys()
        bare_machine = backend['machines'][machine_ids[0]]
        machine_hostname = bare_machine.get('dns_name', None)
        if not machine_hostname:
            machine_hostname = bare_machine['public_ips'][0]

        if not machine_ip:
            machine_ip = machine_hostname
        key = machine_hostname
        machine_name = backend['machines'][machine_ids[0]]['name']
        machine_id = machine_ids[0]

        keypairs = keys.keys()
        for i in keypairs:
            keypair_machines = keys[i]['machines']
            if keypair_machines:
                keypair_machs = keys[i]['machines']
                for mach in keypair_machs:
                    if mach[1] == machine_id:
                        machine_key = i
                        break
            else:
                pass

    client.add_backend(title, provider, key, secret, tenant_name=tenant_name, region=region, apiurl=apiurl,
                       machine_ip=machine_ip, machine_key=machine_key, machine_user=machine_user,
                       compute_endpoint=compute_endpoint, machine_port=machine_port)


def sync_backends(user_dict, client):
    added_backends = user_dict['backends']
    backends = client.backends

    print ">>>Syncing backends"
    for key in added_backends:
        if key in backends.keys():
            print "Found %s" % added_backends[key]['title']
        else:
            print "Adding backend %s" % added_backends[key]['title']
            if added_backends[key]['provider'] == "bare_metal":
                add_bare_metal_backend(client, added_backends[key], user_dict['keypairs'])
            else:
                add_backend(client, added_backends[key])
    print


def add_key(client, key, key_id):
    key_name = key_id
    private = key['private']
    client.add_key(key_name=key_name, private=private)

    if key['default']:
        client.update_keys()
        chosen_key = client.keys[key_id]
        chosen_key.set_default()


def sync_keys(user_dict, client):
    added_keys = user_dict['keypairs']
    keys = client.keys

    print ">>>Syncing keys"
    for key in added_keys:
        if key in keys.keys():
            print "Found %s" % key
        else:
            print "Adding key %s" % key
            add_key(client, key=added_keys[key], key_id=key)
    print


def associate_keys(user_dict, client):
    """
    This whole function is black magic, had to however cause of the way we keep key-machine association
    """
    added_keys = user_dict['keypairs']

    print ">>>Updating Keys-Machines association"
    for key in added_keys:
        machines = added_keys[key]['machines']
        if machines:
            try:
                for machine in machines:
                    backend_id = machine[0]
                    machine_id = machine[1]
                    ssh_user = machine[3]
                    ssh_port = machine[-1]
                    key = client.keys[key]
                    backend = backend_from_id(client, backend_id)
                    backend.update_machines()
                    mach = machine_from_id(backend, machine_id)
                    public_ips = mach.info.get('public_ips', None)
                    if public_ips:
                        host = public_ips[0]
                    else:
                        host = ""
                    key.associate_to_machine(backend_id=backend_id, machine_id=machine_id, host=host, ssh_port=ssh_port,
                                             ssh_user=ssh_user)

                    print "associated machine %s" % machine_id
            except Exception as e:
                pass
    client.update_keys()
    print


def sync(path):

    user_dict = load_dbyaml(path)
    client = init_client()
    sync_keys(user_dict, client)
    sync_backends(user_dict, client)
    associate_keys(user_dict, client)


import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate


def list_providers(client, pretty):
    providers = client.supported_providers

    if pretty:
        x = PrettyTable(["Provider", "Provider ID", "Region", "Region ID"])
        for provider in providers:
            if provider['regions']:
                for region in provider['regions']:
                    x.add_row([provider['title'], provider['provider'], region['location'], region['id']])
            else:
                x.add_row([provider['title'], provider['provider'], '-', '-'])

        print x
    else:
        for provider in providers:
            if provider['regions']:
                for region in provider['regions']:
                    print "%-30s %-20s %-20s %-20s" % \
                          (provider['title'], provider['provider'], region['location'], region['id'])
            else:
                print "%-30s %-20s" % (provider['title'], provider['provider'])


def provider_action(args):

    client = authenticate()

    if args.action == "list-providers":
        pretty = args.pretty
        list_providers(client,pretty)



import os
import sys
import getpass
import time
import ConfigParser
from mistclient import MistClient
from prettytable import PrettyTable


def init_client(mist_uri, email, password):
    try:
        client = MistClient(mist_uri=mist_uri, email=email, password=password)

        #Ensures that GET requests are autheticated
        client.backends
        return client
    except Exception as e:
        print e
        sys.exit(1)


def parse_config():
    home_path = os.getenv("HOME")
    config_path = os.path.join(home_path, ".mist")

    config = ConfigParser.ConfigParser()

    # Set default mist uri
    config.add_section("mist.io")
    config.set("mist.io", "mist_uri", "https://mist.io")

    # Set default credentials
    config.add_section("mist.credentials")
    config.set("mist.credentials", "email", None)
    config.set("mist.credentials", "password", None)

    # Read configuration file
    if os.path.isfile(config_path):
        config.readfp(open(config_path))
    else:
        print "No config file found at: %s" % config_path

    mist_uri = config.get("mist.io", "mist_uri")
    email = config.get("mist.credentials", "email") or prompt_email()
    password = config.get("mist.credentials", "password") or prompt_password()

    if not os.path.isfile(config_path):
        prompt_save_config(mist_uri, email, password, config_path)

    return {
        'mist_uri': mist_uri,
        'email': email,
        'password': password,
    }


def prompt_save_config(mist_uri, email, password, config_path):
    answered = None
    while not answered:
        answer = raw_input("Save config [Y/n]: ")
        if answer in ["y", "Y", "Yes"]:
            answer = True
            answered = True
        elif answer in ["n", "N", "No"]:
            answer = False
            answered = True
        else:
            print "Wrong answer!"

    config_template = """[mist.io]
mist_uri=%s

[mist.credentials]
email=%s
password=%s

""" % (mist_uri, email, password)

    if answer:
        with open(config_path, "w") as f:
            f.write(config_template)
        print "Saved config at %s" % config_path
        return
    else:
        return


def prompt_email():
    return raw_input("Email: ")


def prompt_password():
    return getpass.getpass("Password: ")


def authenticate():
    config = parse_config()
    return init_client(config["mist_uri"], config["email"], config["password"])


def user_info():

    client = authenticate()

    current_plan = client.user_details.get('current_plan')
    user_details = client.user_details.get('user_details')

    print "User Details:"
    x = PrettyTable(user_details.keys())
    x.add_row(user_details.values())
    print x
    print

    print "Current Plan:"
    x = PrettyTable(current_plan.keys())
    expiration = current_plan['expiration']
    current_plan['expiration'] = time.ctime(expiration)
    started = current_plan['started']
    current_plan['started'] = time.ctime(started)
    x.add_row(current_plan.values())
    print x
    print



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

requires = [
    'requests',
    'argparse',
    'argcomplete',
    'pyyaml',
    'prettytable',
    'ansible'
]


def readme():
    with open('README.md') as f:
        return f.read()

setup(
    name='mist',
    version='1.0.0',
    description='Python client for mist.io',
    long_description=readme(),
    classifiers=[
        "Programming Language :: Python",
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
    ],
    url='https://github.com/mistio/mist.client',
    keywords='web cloud server management monitoring automation mobile libcloud pyramid amazon aws rackspace openstack linode softlayer digitalocean gce',
    author='Chris Loukas',
    author_email='commixon@gmail.com',
    license='GPLv3',
    packages=find_packages('src'),
    scripts=['src/mistcommand/mist',
             'src/mistansible/mistplay'],
    package_dir={'': 'src'},
    install_requires=requires,
    zip_safe=False,
    include_package_data=True
)



import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate
from mistcommand.helpers.backends import return_backend


def list_sizes(backend, pretty):
    sizes = backend.sizes

    if pretty:
        x = PrettyTable(["Name", "ID"])
        for size in sizes:
            x.add_row([size['name'], size['id']])

        print x
    else:
        for size in sizes:
            print "%-20s %-20s" % (size['name'], size['id'])


def size_action(args):

    if args.action == 'list-sizes':
        client = authenticate()
        backend = return_backend(client, args)

        pretty = args.pretty
        list_sizes(backend, pretty)


import sys

from prettytable import PrettyTable
from mistcommand.helpers.login import authenticate
from mistcommand.helpers.backends import return_backend


def choose_machines_by_tag(client, tag):
    chosen_machines = []
    machines = client.machines()

    for machine in machines:
        machine_tags = machine.info.get('tags', [])
        if tag in machine_tags:
            chosen_machines.append(machine)

    return chosen_machines


def list_machines(client, backend, pretty):
    x = PrettyTable(["Name", "ID", "State", "Public Ips", "Backend Title", "Tags"])
    if not backend:
        machines = client.machines()
        for machine in machines:
            try:
                public_ips = machine.info['public_ips']
                ips = " -- ".join(public_ips)
            except:
                ips = ""
            machine_tags = machine.info.get('tags', [])
            tags = ",".join(machine_tags)

            if pretty:
                x.add_row([machine.name, machine.id, machine.info['state'], ips, machine.backend.title, tags])
            else:
                print "%-25s %-60s %-10s %-20s %-30s %-20s" % (machine.name, machine.id, machine.info['state'], ips,
                                                               machine.backend.title, tags)

    else:
        machines = backend.machines()
        for machine in machines:
            try:
                public_ips = machine.info['public_ips']
                ips = " -- ".join(public_ips)
            except:
                ips = ""
            machine_tags = machine.info.get('tags', [])
            tags = ",".join(machine_tags)
            if pretty:
                x.add_row([machine.name, machine.id, machine.info['state'], ips, backend.title, tags])
            else:
                print "%-25s %-60s %-10s %-20s %-30s %-20s" % (machine.name, machine.id, machine.info['state'], ips,
                                                               machine.backend.title, tags)

    if pretty:
        print x


def display_machine(machine):
    x = PrettyTable(["Name", "ID", "State", "Public Ips", "Backend Title", "Tags"])

    try:
        public_ips = machine.info['public_ips']
        ips = " -- ".join(public_ips)
    except:
        ips = ""

    machine_tags = machine.info.get('tags', [])
    tags = ",".join(machine_tags)

    x.add_row([machine.name, machine.id, machine.info['state'], ips, machine.backend.title, tags])
    print x


def machine_take_action(machine, action):
    if action == "start":
        machine.start()
        print "Started machine %s" % machine.name
    elif action == "stop":
        machine.stop()
        print "Stopped machine %s" % machine.name
    elif action == "reboot":
        machine.reboot()
        print "Rebooted machine %s" % machine.name
    elif action == "destroy":
        machine.destroy()
        print "Destroyed machine %s" % machine.name
    elif action == "probe":
        info = machine.probe()
        if "uptime" in info.keys():
            print "Machine %s is probed" % machine.name
        else:
            print "Not probed"


def choose_machine(client, args):
    machine_id = args.id
    machine_name = args.name
    if machine_id:
        machines = client.machines(id=machine_id)
        machine = machines[0] if machines else None
    elif machine_name:
        machines = client.machines(name=machine_name)
        machine = machines[0] if machines else None
    else:
        machines = client.machines(search=args.machine)
        machine = machines[0] if machines else None

    return machine


def return_machine(client, args):
    machine_id = args.machine_id
    machine_name = args.machine_name
    if machine_id:
        machines = client.machines(id=machine_id)
        machine = machines[0] if machines else None
    elif machine_name:
        machines = client.machines(name=machine_name)
        machine = machines[0] if machines else None
    else:
        machines = client.machines(search=args.machine)
        machine = machines[0] if machines else None

    return machine


def create_machine(client, backend, args):
    keys = client.keys(id=args.key_id)
    key = keys[0] if keys else None

    if not key:
        print "Could not find key: %s" % args.key_id
        sys.exit(1)

    name = args.machine_name
    image_id = args.image_id
    size_id = args.size_id
    location_id = args.location_id
    networks = []
    if args.network_id:
        networks.append(args.network_id)

    backend.create_machine(name=name, key=key, image_id=image_id, size_id=size_id, location_id=location_id,
                           networks=networks)


def machine_action(args):

    client = authenticate()

    if args.action == 'list-machines':
        if args.backend or args.backend_id or args.backend_name:
            backend = return_backend(client, args)
        else:
            backend = None

        pretty = args.pretty

        list_machines(client, backend, pretty)

    elif args.action == 'describe-machine':
        machine = choose_machine(client, args)

        display_machine(machine)

    elif args.action == 'create-machine':
        backend = return_backend(client, args)
        create_machine(client, backend, args)
        print "Created machine %s" % args.machine_name

    elif args.action in ['start', 'stop', 'reboot', 'destroy', 'probe']:
        machine = choose_machine(client, args)

        machine_take_action(machine, args.action)
    elif args.action == 'enable-monitoring':
        machine = choose_machine(client, args)
        machine.enable_monitoring()
        print "Enabled monitoring to machine %s" % machine.name

    elif args.action == 'disable-monitoring':
        machine = choose_machine(client, args)
        machine.disable_monitoring()
        print "Disabled monitoring to machine %s" % machine.name

    elif args.action == 'tag':
        machine = choose_machine(client, args)

        if machine.info['can_tag']:
            machine.tag(tag=args.new_tag)
            print "Add tag %s to machine %s" % (args.new_tag, machine.name)
        else:
            print "Cannot tag machine on provider %s" % machine.backend.title
            sys.exit(0)


from prettytable import PrettyTable

from mistcommand.helpers.login import authenticate
from mistcommand.helpers.machines import return_machine


def list_metrics(machine, pretty):
    plugins = machine.available_metrics

    x = PrettyTable(["Metric Name", "Metric ID"])

    if pretty:
        for key in plugins.keys():
            plugin = plugins[key]
            x.add_row([plugin['name'], key])

        print x
    else:
        for key in plugins.keys():
            plugin = plugins[key]
            print "%-50s %-20s" % (plugin['name'], key)


def metric_action(args):

    client = authenticate()

    if args.action == 'list-metrics':
        machine = return_machine(client, args)
        pretty = args.pretty
        list_metrics(machine, pretty)
    elif args.action == 'add-metric':
        machine = return_machine(client, args)

        machine.add_metric(args.metric_id)
        print "Added Metric %s to monitored machine %s" % (args.metric_id, machine.name)
    elif args.action == 'add-custom-metric':
        machine = return_machine(client, args)
        name = args.metric_name
        python_file = args.file_path
        value_type = args.value_type
        unit = args.unit

        machine.add_python_plugin(name, python_file, value_type, unit)

        print "Added custom metric %s" % name